# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

class mrp_production(osv.osv):
    _inherit = 'mrp.production'

    def _get_sale_ref(self, cr, uid, ids, field_names=None, arg=False, context=None):
        """ Finds reference of sales order for production order.
        @param field_names: Names of fields.
        @param arg: User defined arguments
        @return: Dictionary of values.
        """
        move_obj = self.pool.get('stock.move')

        def get_parent_move(move_id):
            move = move_obj.browse(cr, uid, move_id)
            if move.move_dest_id:
                return get_parent_move(move.move_dest_id.id)
            return move_id

        res = {}
        productions = self.browse(cr, uid, ids)
        for production in productions:
            res[production.id] = {}
            if production.move_prod_id:
                parent_move_line = get_parent_move(production.move_prod_id.id)
                if parent_move_line:
                    move = move_obj.browse(cr, uid, parent_move_line)
                    for field_name in field_names:
                        if field_name == 'sale_name':
                            res[production.id][field_name] = move.sale_line_id and move.sale_line_id.order_id.name or False
                        if field_name == 'sale_ref':
                            res[production.id][field_name] = move.sale_line_id and move.sale_line_id.order_id.client_order_ref or False
                        if field_name == 'sale_id':
                            res[production.id][field_name] = move.sale_line_id and move.sale_line_id.order_id.id or False
                        if field_name == 'sale_line_id':
                            res[production.id][field_name] = move.sale_line_id and move.sale_line_id.id or False
                        if field_name == 'partner_id':
                            res[production.id][field_name] = move.sale_line_id and move.sale_line_id.order_id.partner_id.id or False
        return res

    _columns = {
        'sale_name': fields.function(_get_sale_ref, method=True, multi='sale_name', type='char', size=64, string='Sales Name', help='Indicate the name of sales order.', store=True),
        'sale_ref': fields.function(_get_sale_ref, method=True, multi='sale_name', type='char', size=64, string='Sales Reference', help='Indicate the Customer Reference from sales order.', store=True),
        'sale_id': fields.function(_get_sale_ref, method=True, multi='sale_name', type='many2one', relation='sale.order', string='Sale Order', help='', store=True),
        'sale_line_id': fields.function(_get_sale_ref, method=True, multi='sale_name', type='many2one', relation='sale.order.line', string='Sale Line', help='', store=True),
        'partner_id':  fields.function(_get_sale_ref, method=True, multi='sale_name', type='many2one', relation='res.partner', string='Partner', help='', store=True),
    }

mrp_production()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
