# -*- encoding: utf-8 -*-
###########################################################################
#    Module Writen to OpenERP, Open Source Management Solution
#
#    Copyright (c) 2010 moylop260 - http://moylop.blogspot.com/
#    All Rights Reserved.
#    info moylop260 (moylop260@hotmail.com)
############################################################################
#    Coded by: moylop260 (moylop260@hotmail.com)
############################################################################
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
import netsvc
import time

class mrp_production(osv.osv):
    _inherit = 'mrp.production'
    _order = 'name'
    
    def _get_sale_id(self, cr, uid, ids, field_names=False):
        move_obj = self.pool.get('stock.move')

        def get_parent_move(move_id):
            move = move_obj.browse(cr, uid, move_id)
            if move.move_dest_id:
                return get_parent_move(move.move_dest_id.id)
            return move_id

        res = {}
        productions = self.browse(cr, uid, ids)
        for production in productions:
            res[production.id] = {}
            if production.move_prod_id:
                parent_move_line = get_parent_move(production.move_prod_id.id)
                if parent_move_line:
                    move = move_obj.browse(cr, uid, parent_move_line)
                    res[production.id]['sale_id'] = move.sale_line_id and move.sale_line_id.order_id.id or False
                    res[production.id]['sale_line_id'] = move.sale_line_id and move.sale_line_id.id or False
        return res
    
    def _split_production(self, cr, uid, ids):
        wf_service = netsvc.LocalService("workflow")
        sale_line_obj = self.pool.get('sale.order.line')
        production_id__sale_id = self._get_sale_id(cr, uid, ids)
        #print "action_confirm", ids
        for production in self.browse(cr, uid, ids):
            #sale_id = production_id__sale_id[id]['sale_id']
            
            additional_production_ids = []
            #print "production.id",production.id
            sale_line_id = production_id__sale_id[production.id].get('sale_line_id', False)
            #print "sale_line_id",sale_line_id
            if sale_line_id: 
                sale_line = sale_line_obj.browse(cr, uid, [sale_line_id])[0]
                
                date_planned_start = sale_line.date_planned_start
                product_qty_split = sale_line.product_qty_split
                #print "date_planned_start",date_planned_start
                self.write(cr, uid, [production.id], {'date_planned': date_planned_start})
                
                while production.product_qty > product_qty_split:
                    new_production_id = self.copy(cr, uid, production.id, {
                        'product_qty': product_qty_split,
                        'product_uos_qty': product_qty_split,
                    })
                    wf_service.trg_validate(uid, 'mrp.production', new_production_id, 'button_confirm', cr)
                    additional_production_ids.append( new_production_id )
                    production.product_qty -= product_qty_split
                self.write(cr, uid, [production.id],  {
                    'product_qty': production.product_qty,
                    'product_uos_qty': production.product_qty,
                })
        return additional_production_ids
    
    def set_local_number(self, cr, uid, ids, context={}):
        """
        production_id__sale_id = self._get_sale_id(cr, uid, ids)
        sale_line_obj = self.pool.get('sale.order.line')
        for production in self.browse(cr, uid, ids):
            sale_line_id = production_id__sale_id[production.id].get('sale_line_id', False)
            if sale_line_id:
                sale_line = sale_line_obj.browse(cr, uid, [sale_line_id])[0]
                sale_line.
        """
        count = 0
        for production in self.browse(cr, uid, ids):
            count += 1
            date_obj = time.strptime(production.date_planned, '%Y-%m-%d %H:%M:%S')
            year = int(time.strftime('%y', date_obj))
            #week_of_year = int(time.strftime('%U', date_obj))+1
            week_of_year = int(time.strftime('%W', date_obj))+1
            week_of_year_str = '%%0%sd' % 2 % week_of_year
            day_of_week = date_obj.tm_wday + 1
            product_code = production.product_id.default_code or ''
            qty = "%%0%sd"% 2 % production.product_qty
            lot = '%%0%sd' % 3 % count
            
            new_name = ""
            new_name += "%s"%(year)
            #new_name += "-"
            new_name += "%s"%(week_of_year_str)
            #new_name += "-"
            new_name += "%s"%(day_of_week)
            #new_name += "-"
            new_name += "%s"%( product_code )
            #new_name += "-"
            new_name += "%s"%( qty )
            #new_name += "-"
            new_name += lot
            self.write(cr, uid, [production.id], {'name': new_name})
        return True
    
    
    def action_confirm(self, cr, uid, ids):
        additional_production_ids = self._split_production(cr, uid, ids)
        self.action_compute(cr, uid, ids, properties=[])
        ret = super(mrp_production, self).action_confirm(cr, uid, ids)
        self.set_local_number(cr, uid, ids + additional_production_ids)
        return ret
mrp_production()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: